'use strict';

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildRegistration = (0, _babelTemplate2.default)('\n  tagSource(ID, NAME);\n');
var buildTagger = (0, _babelTemplate2.default)('\n(function () {\n  function tagSource(fn, localName) {\n    if (typeof fn !== "function") {\n      return;\n    }\n\n    if (fn.hasOwnProperty("__source")) {\n      return;\n    }\n\n    try {\n      Object.defineProperty(fn, "__source", {\n        enumerable: false,\n        configurable: true,\n        value: {\n          fileName: FILENAME,\n          localName: localName\n        }\n      });\n    } catch (err) {}\n  }\n  REGISTRATIONS\n})();\n');

module.exports = function (args) {
  if (this && this.callback) {
    throw new Error('React Hot Loader: You are erroneously trying to use a Babel plugin ' + 'as a Webpack loader. We recommend that you use Babel, ' + 'remove "react-hot-loader/babel" from the "loaders" section ' + 'of your Webpack configuration, and instead add ' + '"react-hot-loader/babel" to the "plugins" section of your .babelrc file. ' + 'If you prefer not to use Babel, replace "react-hot-loader/babel" with ' + '"react-hot-loader/webpack" in the "loaders" section of your Webpack configuration. ');
  }
  var t = args.types;


  if (process.env.NODE_ENV === 'production') {
    return { visitor: {} };
  }

  function shouldRegisterBinding(binding) {
    var _binding$path = binding.path;
    var type = _binding$path.type;
    var node = _binding$path.node;

    switch (type) {
      case 'FunctionDeclaration':
      case 'ClassDeclaration':
      case 'VariableDeclaration':
        return true;
      case 'VariableDeclarator':
        var init = node.init;

        if (t.isCallExpression(init) && init.callee.name === 'require') {
          return false;
        }
        return true;
      default:
        return false;
    }
  }

  var DEFAULT_EXPORT = Symbol();
  return {
    visitor: {
      ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
        if (path.node.declaration.id) {
          return;
        }

        var id = path.scope.generateUidIdentifier('default');
        var expression = t.isExpression(path.node.declaration) ? path.node.declaration : t.toExpression(path.node.declaration);

        path.insertBefore(t.variableDeclaration('const', [t.variableDeclarator(id, expression)]));
        path.node.declaration = id;
        path.parent[DEFAULT_EXPORT] = id;
      },


      Program: {
        enter: function enter(_ref) {
          var node = _ref.node;

          node[DEFAULT_EXPORT] = null;
        },
        exit: function exit(_ref2, _ref3) {
          var node = _ref2.node;
          var scope = _ref2.scope;
          var file = _ref3.file;

          var registrations = [];

          for (var id in scope.bindings) {
            var binding = scope.bindings[id];
            if (shouldRegisterBinding(binding)) {
              registrations.push(buildRegistration({
                ID: binding.identifier,
                NAME: t.stringLiteral(id)
              }));
            }
          }

          if (node[DEFAULT_EXPORT]) {
            registrations.push(buildRegistration({
              ID: node[DEFAULT_EXPORT],
              NAME: t.stringLiteral('default')
            }));
          }

          node.body.push(buildTagger({
            FILENAME: t.stringLiteral(file.opts.filename),
            REGISTRATIONS: registrations
          }));
        }
      }
    }
  };
};