'use strict';

var React = require('react');
var createProxy = require('react-proxy').default;

var proxies = {};
function resolveType(type) {
  if (!type) {
    return type;
  }
  if (!Object.hasOwnProperty.call(type, '__source')) {
    return type;
  }
  var source = type.__source;
  if (!source || !source.fileName || !source.localName) {
    return type;
  }

  var fairlyUniqueID = source.fileName + '#' + source.localName;
  if (!proxies[fairlyUniqueID]) {
    proxies[fairlyUniqueID] = createProxy(type);
  } else {
    proxies[fairlyUniqueID].update(type);
  }
  return proxies[fairlyUniqueID].get();
}

if (React.createElement.isPatchedByReactHotLoader) {
  throw new Error('Cannot patch React twice.');
}

var createElement = React.createElement;
function patchedCreateElement(type) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return createElement.apply(undefined, [resolveType(type)].concat(args));
}
patchedCreateElement.isPatchedByReactHotLoader = true;
React.createElement = patchedCreateElement;